#include <avr/io.h>
#include <string.h>
#include <stdlib.h>
#include <util/delay.h>
#include "uart.h"

#include "protocol.h"
int toggle = 1;
/** Initialize ports (input/output) **/
void initPorts()
{
	//Set port B on output
	DDRB  = 0xff;
	PORTB = 0xff;

	//Set port C on output
	PORTC = 0xff;
	DDRC  = 0x00;
	
	//Set port D on input
	DDRD  = 0xfc;
	PORTD = 0x00;

	//Analog to digital setup
	ADCSRA = _BV(ADEN) | _BV(ADPS2) | _BV(ADPS1) | _BV(ADPS0); //Enable ADC and set 
}

/** Start handling for Ipemotion protocol **/
void startProtocolHanlde()
{
	unsigned char com_buffer[MSG_SIZE];
	unsigned char cmd[CMD_SIZE];

	while(1)
	{
		uart_gets(com_buffer, MSG_SIZE);
		strncpy (cmd, com_buffer, CMD_SIZE);
		#ifdef debug
			uart_puts("Recieved Message: ");
			uart_puts(com_buffer);
		#endif

		if(strncmp(cmd,"RDI", CMD_SIZE) == 0)
			readDigitalIn(com_buffer);

		else if(strncmp(cmd, "RDO", CMD_SIZE) == 0)
			readDigitalOut(com_buffer);

		else if(strncmp(cmd, "RAO", CMD_SIZE) == 0)
			readAnalogOut(com_buffer);

		else if(strncmp(cmd, "RAI", CMD_SIZE) == 0)
			readAnalogIn(com_buffer);

		else if(strncmp(cmd, "SAO", CMD_SIZE) == 0)
			setAnalogOut(com_buffer);

		else if(strncmp(cmd,"SDO", CMD_SIZE) == 0)
			setDigitalChannel(com_buffer);
		else
		{
			uart_flush();
			_delay_ms(80);
			uart_flush();
		}
	}
}

/** Set digital output channel **/
int setDigitalChannel(unsigned char *str)
{
	unsigned char ret = -1;

	unsigned char data[MSG_SIZE - CMD_SIZE];
	strncpy(data, str + CMD_SIZE, MSG_SIZE);

	int nr		= (int)data[0] - 48;
	int value 	= (int)data[1] - 48;

	char buffer [33];
	
	nr = nr - 1;
	if((nr < 0) || (nr > 15))
	{
		uart_puts("Error: Port not Valid!");
		return -1;
	}
	
	if(nr < 8)	//PORT B
	{
		if (value==0)
		   PORTB &= ~(1 << nr);
		else
		    PORTB |= (1 << nr);
	}
	else if(nr < 16) //PORT C
	{
		if (value==0)
			PORTC &= ~(1 << nr%8);
		else
		   	PORTC |= (1 << nr%8);
	}
	uart_puts(itoa(value,buffer,10));

	return ret;
}

/** Read digital input channel **/
int readDigitalIn(unsigned char *str)
{
	unsigned char ret = -1;

	unsigned char data[MSG_SIZE - CMD_SIZE];
	strncpy(data, str + CMD_SIZE, MSG_SIZE);

	int nr		= (int)data[0] - 48;

	char buffer [33];

	int result = 0;

	nr = nr + 1;
	if(nr < 8)
		uart_puts( itoa(result,buffer,10));
	else
	{
		uart_puts("Error: Port not Valid!");
	}
	result = ( ( PIND & ( 1 << nr ))  >> nr);

	return ret;
}

/** Read digital output channel **/
int readDigitalOut(unsigned char *str)
{
	unsigned char ret = -1;

	unsigned char data[MSG_SIZE - CMD_SIZE];
	strncpy(data, str + CMD_SIZE, MSG_SIZE);

	int nr		= (int)data[0] - 48;

	char buffer [33];

	int result = 0;

	nr = nr - 1;
	if((nr < 0) || (nr > 15))
	{
		uart_puts("Error: Port not Valid!");
		return -1;
	}

	if(nr < 8)	//PORT B
	{
		result = ( ( PINB & ( 1 << nr ))  >> nr);
	}
	else if(nr < 16) //PORT C
	{
		nr = nr%8;
		result = ( ( PINC & ( 1 << nr ))  >> nr);
	}
	else
	{
		uart_puts("Error: Port not Valid!");
		return -1;
	}
	uart_puts( itoa(result,buffer,10));

	return ret;
}

/** Read analog input**/
int readAnalogIn(unsigned char *str)
{
	unsigned char ret = -1;

	unsigned char data[MSG_SIZE - CMD_SIZE];
	strncpy(data, str + CMD_SIZE, MSG_SIZE);

	int nr		= (int)data[0] - 48;

	char buffer [33];
	int result;
	nr = nr - 1;

	if((nr >= 0) && (nr < 8))	//Check if the channel number is in valid range
	{
		ADMUX = 0;
		ADMUX |= (ADMUX & ~(0x1F)) | (nr & 0x1F);               // Set channel
	   	ADCSRA |= _BV(ADSC);               // Start converting
	   	while(!bit_is_set(ADCSRA,ADIF));   // Loop until converting is done
	   	ADCSRA |= _BV(ADIF);               // Clear ADIF by setting to 1

		result = (ADC);
	}
	else
	{
		uart_puts("Error: Port not Valid!");
		return -1;
	}

	if(result < 10)
	{	//9
		uart_puts( itoa(result,buffer,10));
		uart_putc('\0');
		uart_putc(' ');
		uart_putc(' ');
	}

	else if(result >= 10 && result < 100)
	{	//99
		uart_puts( itoa(result,buffer,10));
		uart_putc('\0');
		uart_putc(' ');
	}
	else if(result >= 100 && result < 1000)
	{	//999
		uart_puts( itoa(result,buffer,10));
		uart_putc('\0');
	}
	else	//1200
		uart_puts( itoa(result,buffer,10));

	return ret;
}

/** Read anlog output **/
int readAnalogOut(unsigned char *str)
{
	unsigned char ret = -1;

	unsigned char data[MSG_SIZE - CMD_SIZE];
	strncpy(data, str + CMD_SIZE, MSG_SIZE);

	return ret;
}

/** Set analog output **/
int setAnalogOut(unsigned char *str)
{
	unsigned char ret = -1;

	unsigned char data[MSG_SIZE - CMD_SIZE];
	strncpy(data, str + CMD_SIZE, MSG_SIZE);

	return ret;
}